<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC2/class/dao/extended/zone.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Zack Bartel   <bartel@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: zone.php 4588 2006-07-07 19:50:03Z morgabra $

/**
  * zone.php - contains the {@link Zone} class
  *
  * @author $Author: morgabra $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

include_once DAODIR.'/Zone.php';

/**
  * A Zone (used for DNS and permissions)
  *
  * @package    classes
  * @subpackage dao
  */
class Zone extends DAO_Zone {

    // {{{ constructor()
    function Zone ( $zone_id = 0 ) 
    {
        global $conf;

        if ($zone_id) {
            $this->id = $zone_id;
            $this->get($this->id);
        }

    } // constructor
    // }}}

// DBDO overloads
    // {{{ delete()

    /**
      * Delete this zone, all associated mappings, and objects
      */
    function delete($useWhere = false) 
    {
        global $conf;

        // make sure the id is set
        if (!isset($this->id) || $this->id < 1) {
            return false;
        }

        // purge the zone (deletes all pointers and hosts)
        $this->purge_zone();

        /* delete all mappings to this zone */
        // mappings (zone_range, mx, etc.) to delete
        $this->_delete_maps();

        // finally, delete the zone
        $result = parent::delete($useWhere);
        if (!$result || PEAR::isError($result)) {
            return $result;
        }

        return $result;
        
    } //delete_zone

    // }}}

// Setters
    // {{{ setAll()
    function setAll()
    {
        $this->setNotify($this->notify);
        $this->setPurgeable($this->purgeable);
        $this->setAllow_blank_ha($this->allow_blank_ha);
    }
    // }}}
    // {{{ setNotify()
    function setNotify($notify) 
    {
        if ($notify === 'disabled' || $notify === false || $notify == 0) {
            $this->notify = 0;
        } else {
            $this->notify = 1;
        }

        return true;
    }
    // }}}
    // {{{ setPurgeable()

    /**
      * Set the 'purgeable' flag for this zone
      *
      * Called without args, the flag is reversed, otherwise,
      * sets the flag as specified
      *
      * @param  mixed   $value  'true'/'false', 0/1, true/false
      * @return bool    true on success
      */
    function setPurgeable ($value = 0) 
    {
        global $conf;

        if ($value == 0) {
            $this->purgeable = 0;
        } else {
            $this->purgeable = 1;
        }

        return true;

    } //setPurgeable
    // }}}
    // {{{ setAllow_blank_ha()
    
    /**
      * Set the 'allow blank hardware address' flag for this zone
      *
      * @param  mixed   true/false, 0/1, 'true'/'false'
      * @return bool    true on success
      */
    function setAllow_blank_ha($allow)
    {
        if ($allow === true || $allow === 'true' || $allow == 1) {
            $this->allow_blank_ha = 1;
        } else {
            $this->allow_blank_ha = 0;
        }

        return true;
    }
    // }}}

// Validators
    // {{{ validateName()

    /**
      * Ensure zone name is valid
      *
      * @return mixed   true on valid, error string on invalid
      */
    function validateName() 
    {
        global $conf;

        // make sure the name is set
        if (!$this->name) {
            return _("Name must be set for this zone");
        }

        // make sure a zone with this name doesn't exist
        // $sql = "SELECT name FROM zone WHERE name LIKE '$name' AND id!='$zone_id'";
        $dbdo = DB_DataObject::Factory('zone');
        $dbdo->selectAdd();
        $dbdo->selectAdd("name");
        $dbdo->whereAdd("name LIKE '$this->name'");
        if ($this->id)
            $dbdo->whereAdd("id!='$this->id'");

        if ($dbdo->find()) {
            return "$this->name "._("already exists as a Zone in the database");
        }

        // make sure that there aren't any invalid chars
        // we can allow alpha-numeric words and spaces
        if (!(preg_match("/^[\s\w\.\_\-]+$/", $this->name))) {
            return "$this->name "._("is an invalid Zone name");
        }

        return true;

    } // validateName

    // }}}

// Maintain overloads
    // {{{ _history_text()
    function _history_text()
    {
        // : ZONE_NAME
        return $this->name;
    }
    // }}}

// Built-in functions
    // {{{ get_confused_hosts()

    /**
      * Get the hosts incorrectly registered by comparing the
      * host_history table versus the specified values
      *
      * @param  int     timestamp
      * @param  int     number of days to offset (value appended to timestamp)
      * @return array   host IDs of confused hosts
      */
    function get_confused_hosts($date,$day_count=1) 
    {
        global $conf;

        // First get all the ranges in this zone
        $ranges = $this->ip_ranges();
        
        // Add restriction to only $date
        $end_date = $date + ( DAY * $day_count );

        // Create the sql statement from the ip ranges to pull any MAC's inside 
        // the ranges
        foreach ($ranges as $range_id) {
            $range = new Range($range_id);
            $end = $range->end->ip_int;
            $start = $range->start->ip_int;

            $sql = "SELECT DISTINCT(host_history.ha),host_history.ip,ranges.type,ranges.id as rangeid FROM host_history,ranges WHERE " .
                   "host_history.ip >= ranges.start AND host_history.ip <= ranges.end AND host_history.ip <= '$end' AND host_history.ip >= '$start'" .
                   " AND host_history.date >= '$date' AND host_history.date <= '$end_date'";
        
            $db_results = mysql_query($sql, dbh());
            //go through the results...
            while ($results = mysql_fetch_object($db_results)) { 
                $host_history[] = $results;
            } //end while
    
        } //end foreach


        // get out if no confused hosts were found
        if (!count($host_history)) {
            return false;
        }

        // Now recurse through and check them... once we are done 
        // we want to only return the incorrectly set ones
        foreach ($host_history as $host) {

            if ($host->type == "dynamic" OR $host->type == "ddns") {
                $sql = "SELECT id FROM host WHERE ha='$host->ha' AND dynamic_range='$host->rangeid'";
            }
            else { 
                $sql = "SELECT id FROM host WHERE ha='$host->ha' AND ip='$host->ip'";
            }

            $db_results = mysql_query($sql, dbh());

            // If we didn't find anything
            if (!$r = mysql_fetch_object($db_results)) {
                $results[] = $host->id;
            } //if not found

        } //end foreach

        return $results;

    }

    // }}}
    // {{{ ip_ranges()

    /**
      * Returns a list of all enabled ranges mapped to this zone,
      * sorted by starting address
      *
      * @return array   IDs (primary key) for Range objects
      */
    function ip_ranges() 
    {
        global $conf;

        /*
         * "SELECT ranges.id FROM ranges,zone_range
         * WHERE ranges.id = zone_range.range AND zone_range.zone='$this->id' 
         * AND ranges.enabled='1' AND zone_range.enabled='1' 
         * ORDER BY ranges.start ASC";
         */

        // create/initialize the ranges object
        $ranges = DB_DataObject::Factory('ranges');
        $ranges->selectAdd();
        $ranges->selectAdd('ranges.id');
        $ranges->enabled = 1;
        $ranges->orderBy("start ASC");

        // create/initialize the zone_range object
        $zone_range = DB_DataObject::Factory('zone_range');
        $zone_range->zone = $this->id;
        $zone_range->enabled = 1;

        // join the zone_range to ranges, and search
        $ranges->joinAdd($zone_range);
        $ranges->find();

        $results = array();
        while ($ranges->fetch()) {
            $results[] = $ranges->id;
        }

        return $results;

    }
    // }}}
    // {{{ purge_zone()

    /**
      * Delete all Hosts and Pointers from this Zone
      *
      * Requires 'purgeable' flag
      *
      * @return mixed   number of hosts and pointers deleted or false if not purgeable
      */
    function purge_zone() 
    {
        global $conf;

        //let's double check to make sure that this zone can be purged         
        if ($this->purgeable == true) {

            // keep a list of the total number of deleted objects
            $sum = 0;

            /* Delete the Hosts */

            // $sql = "DELETE FROM host WHERE zone='$this->id'";
            $dbdo = DB_DataObject::Factory('host');
            $dbdo->zone = $this->id;
            $dbdo->find();

            // one host at a time for object_history's sake
            while ($dbdo->fetch()) {
                $host = new Host($dbdo->id);
                $sum += $host->delete();
            }

            /* Delete the Pointers */

            // $sql = "DELETE FROM pointer WHERE zone='$this->id'";
            $dbdo = DB_DataObject::Factory('pointer');
            $dbdo->zone = $this->id;
            $dbdo->find();

            // one pointer at a time for object_history's sake
            while ($dbdo->fetch()) {
                $pointer = new Pointer($dbdo->id);
                $sum += $pointer->delete();
            }

            return $sum;
        }

        return false;

    } //purge_zone

    // }}}
    // {{{ add_zone_user()

    /**
      * Add a user to this zone with specified access level
      *
      * @param  string  $username   user to add
      * @param  int     $level      access level for user
      * @return mixed   true on success, PEAR_Error on failure
      */
    function add_zone_user ($username, $level) 
    {
        global $conf;

        $usr = DB_DataObject::Factory('user');
        
        // grab the user's info or create a new user entry if needed
        if (!$usr->get($username)) {

            // get the auth type for given username, or error if doesn't exist
            $type = User::check_auth_name($username);
            if (!$type) {
                return new PEAR_Error(_("User doesn't exist"));
            }

            $usr = new User($username);
            $usr->setType($type);
            $usr->setZone($this->id);
            $usr->setPreferred_zone($this->id);
            if (!$usr->insert()) {
                return new PEAR_Error(_("Could not create user"));
            }
        }

        $usr = new User($username);

        // make sure they don't already have a mapping in this zone
        if ($usr->current_zone == $this->id AND isset($usr->level)) {
            return _("User already exists in Zone"). " $this->name";
        }

        // set the user's permission for this zone
        $usr->setLevel($level, $this->id);

        // validate it
        $valid = $usr->validate();

        if ($valid !== true) {
            // ok, we need to format the list of errors

            // first, strip out all valid results
            $errors = array();
            foreach ($valid as $var=>$error) {
                if ($error !== true)
                    $errors[] = ucfirst($var)." => $error";
            }

            // next, start creating the error string
            $res = _("The following errors occured while trying to add a user to this zone:");
            $res .= '  '.implode(',', $errors);

            return $res;
        }

        // if we're this far, let's set the mapping
        if (!($res = $usr->update())) {
            return "$res";
        }

        return true;
       
    } // add_zone_user
    // }}}
    // {{{ get_zone_history()

    /**
      * Get the delete records, and all history for this zone
      *
      * @return object  object with an array for each type of zone object
      *                 holding a list of object_history IDs
      */
    function get_zone_history() 
    {
        
        // First let's get the deleted records

        // "SELECT * FROM object_history WHERE object_type='delete' AND object_id='$this->id' ORDER BY object_history.date DESC"
        $dbdo = DB_DataObject::Factory('object_history');
        $dbdo->object_type = 'delete';
        $dbdo->object_id = $this->id;
        $dbdo->orderBy('date DESC');
        $count = $dbdo->find();

        while ($dbdo->fetch())
            $results->delete[] = $dbdo->id;

        // Second let's collect the hosts.. this is a littler harder

        $sql = "SELECT host.id FROM object_history,host WHERE host.id=object_history.object_id AND object_history.object_type='host' AND host.zone='$this->id'";
        $sql .= " ORDER BY object_history.date DESC";
        $db_results = mysql_query($sql, dbh());

        while ($r = mysql_fetch_object($db_results))
            $results->host[] = $r->id;
            
        // These are cnames

        $sql = "SELECT cname.id FROM object_history,cname WHERE object_history.object_type='cname' AND object_history.object_id=cname.id AND  cname.zone='$this->id'";
        $sql .=" ORDER BY object_history.date DESC";
        $db_results = mysql_query($sql, dbh());
    
        while ($r = mysql_fetch_object($db_results))
            $results->cnames[] = $r->id;

        // Next let's get the mx records
        $sql = "SELECT mx.id FROM object_history,mx WHERE object_history.object_type='mx' AND object_history.object_id=mx.id AND mx.zone='$this->object_id'";
        $sql .= " ORDER BY object_history.date DESC";
        $db_results = mysql_query($sql, dbh());

        while ($r = mysql_fetch_object($db_results))
            $results->mxs[] = $r->id;

        return $results;

    } // get_zone_history

    // }}}
    // {{{ get_next_host_by_ip()

    /**
    *
    * Return the next IP in the zone
    * For this host and zone, return the next host
    * object for this zone by IP address.
    *
    * @access    public
    * @param    Host    Host or DAO_Host object, with populated data
    * @return    mixed   int id of next host, PEAR_Error on error
    */
    function get_next_host_by_ip($host) 
    {

        // make sure a populated host object is passed
        if (!is_object($host) || !isset($host->id)) {
            return new PEAR_Error(
                    _("Invalid Host or DAO_Host object passed"));
        }

        // make sure this zone is instantiated
        if (!isset($this->id)) {
            return new PEAR_Error(__FUNCTION__ .' '.
                    _("must be called from an instantiated Zone object"));
        }
                    
        // new host object for querying
        $dbdo = DB_DataObject::Factory('host');

        if ($host->ip) {
            /* if this is a static host then simply get the next possible ip address (gets next in line...) */

            // increment the IP addr and make new object
            $ip_int = $host->ip;
            $ip_int++;
    
            //All we have to do is order by ip on the host table
            //let's keep it in the same zone though

            // "SELECT * FROM host WHERE ip>='$ip_int' AND zone='$zone_id' ORDER BY ip LIMIT 1"
            $dbdo->zone = $this->id;
            $dbdo->whereAdd("ip>='$ip_int'");
            $dbdo->orderBy('ip');
        } elseif ($host->dynamic_range) {
            /* if this is a dynamic host then get the next dynamic host by id */

            //we are dealing with a dynamic address.... 
            // "SELECT * FROM host WHERE dynamic_range='$this->dynamic_range' AND zone='$this->zone' AND id!='$this->id' ORDER BY id LIMIT 1"
            $dbdo->dynamic_range = $host->dynamic_range;
            $dbdo->zone =  $this->id;
            $dbdo->whereAdd("id!='$host->id'");
            $dbdo->orderBy('id');
        } else {
            /* make sure either an ip or dynamic_range is set */

            return new PEAR_Error(
                _("Cannot obtain next host by IP without specifying the base IP/Range"));
        }

        $dbdo->find(true);

        return $dbdo->id;
    } //get_next_host_by_ip

    // }}}
    // {{{ get_next_host_by_name()

    /**
    * 
    * Return the next host id for this zone. 
    *
    * @param    Host    Host or DAO_Host object (populated)
    * @return    mixed    The ID of the next host in the zone.
    *                    If there isn't another host return null
    * @access    public
    */
    function get_next_host_by_name($host) 
    {
        // make sure a populated host object is passed
        if (!is_object($host) || !isset($host->id)) {
            return new PEAR_Error(
                    _("Invalid Host or DAO_Host object passed"));
        }

        // make sure this zone is instantiated
        if (!isset($this->id)) {
            return new PEAR_Error(__FUNCTION__ .' '.
                    _("must be called from an instantiated Zone object"));
        }
 
        // "SELECT id FROM host WHERE zone=$zone_id ORDER BY name"
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->zone = $this->id;
        $dbdo->orderBy('name');
        $dbdo->find();
        while ($dbdo->fetch()) {
            if ($dbdo->id == $host->id) {
                $dbdo->fetch();
                return $dbdo->id;
            }
        }

        return null;
    } // get_next_host_by_name

    // }}}
    // {{{ get_workgroups()
    /**
    *
    * Returns the workgroups the host could belong to
    *
    * This function discovers the workgroups a host CAN belong, not the 
    * workgroups it belongs to.  A VERY subtle difference.
    *
    * This function may be called statically or not
    *
    * @param    integer $zone_id Defaults to the hosts zone
    * @return    array   list of workgroup IDs (primary keys)
    *                   or PEAR_Error if no zone specified
    * @access    public
    * @see        Workgroup
    */
    function get_workgroups($zone_id = 0) 
    {
        global $conf;

        if (!$zone_id) {
            if ($this->id) {
                $zone_id = $this->id;
            } else {
                return new PEAR_Error(_("No zone specified for").' '.__FUNCTION__);
            }
        }

        /* Get all workgroups for specified zone */

        // "SELECT workgroup FROM zone_workgroup WHERE zone_workgroup.zone='$zone_id'"
        $dbdo = DB_DataObject::Factory('zone_workgroup');
        $dbdo->selectAdd();
        $dbdo->selectAdd('workgroup');
        $dbdo->zone = $zone_id;
        $dbdo->find();

        $results = array();
        while ($dbdo->fetch()) {
            $results[] = $dbdo->workgroup;
        }

        /* Get the default workgroup(s) */
        // SELECT * FROM workgroup WHERE name LIKE $conf['general']['default_workgroup']
        $dbdo = DB_DataObject::Factory('workgroup');
        $dbdo->name = $conf['general']['default_workgroup'];
        $dbdo->find();
        
        $default_group = array();
        while ($dbdo->fetch())
            $default_group[] = $dbdo->id;

        $results = array_merge($default_group,$results);
        
        return $results;

    } // get_workgroups

    // }}}
    // (global getters)
    // {{{ get_host_os()

    /**
      * Return an array of the names of every OS listed in any Host in this zone
      *
      * @return array   list of OS names
      */
    function get_host_os() 
    {
        global $conf;

        // "SELECT distinct(os) FROM host WHERE zone='$this->id' ORDER BY os ASC";
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();
        $dbdo->selectAdd("DISTINCT(os) as os");
        $dbdo->zone = $this->id;
        $dbdo->orderBy("os ASC");
        $dbdo->find();

        $results = array();
        while ($dbdo->fetch()) {
            $results[] = $dbdo->os;
        }

        return $results;

    } // get_host_os

    // }}}
    // {{{ get_host_hardware_type()

    /**
      * Get a list of all hardware tyeps in this zone
      *
      * @return array   array (string) of all hardware types any host 
      *                 is using in this zone
      */
    function get_host_hardware_type() 
    {
        global $conf;

        $dbdo = DB_DataObject::Factory('host');
        $dbdo->selectAdd();
        $dbdo->selectAdd("DISTINCT(type) as type");
        $dbdo->zone = $this->id;
        $dbdo->orderBy("type ASC");
        $dbdo->find();

        $results = array();
        while ($dbdo->fetch()) {
            $results[] = $dbdo->type;
        }

        return $results;

    } // get_host_hardware_type

    // }}}
    // {{{ get_unregistered_hosts()
    //
    // get any hosts which are in the host_history table but 
    // are not registered in maintain.. this is kinda tricky
    // grabs from $this (get's all in zone)
    function get_unregistered_hosts($date,$day_count=0) 
    {
        global $conf;

        // First get all the ranges in this zone
        $ranges = $this->ip_ranges();
        
        // Create the sql statement from the ip ranges to pull any MAC's inside 
        // the ranges
        foreach ($ranges as $range_id) {
            $range = new Range($range_id);
            $end = $range->end->ip_int;
            $start = $range->start->ip_int;
            if (!$sql) { $sql = "SELECT DISTINCT(host_history.ha),host_history.ip FROM host_history WHERE ( host_history.ip <= '$end' AND host_history.ip >= '$start' )"; }
            else { $sql .= " OR ( host_history.ip <= '$end' AND host_history.ip >= '$start') "; }

        } //end foreach

        // Add restriction to only $date
        if (!$day_count) { $day_count = 1; }
        $end_date = $date + ( DAY * $day_count );
        $sql .= " AND host_history.date >= '$date' AND host_history.date <= '$end_date'";

        // Add a limit so it only shows the latest date field
        
        $db_results = mysql_query($sql, dbh());

        //go through the results...
        while ($results = mysql_fetch_object($db_results)) { 
            $host_history[] = $results;
        } //end while
        
        if (!count($host_history)) { return false; }

        // Now recurse through and check them... once we are done 
        // we want to only return the unregistered ones
        foreach ($host_history as $host) {

            $sql = "SELECT id FROM host WHERE ha='$host->ha'";
            $db_results = mysql_query($sql, dbh());

            // If we didn't find anything
            if (!$r = mysql_fetch_object($db_results)) {
                $results[] = $host;
            } //if not found

        } //end foreach

        return $results;

    } //get_unregistered_hosts
    // }}}
    // {{{ get_users()

    /**
      * Get the list of users who have any level of access to this zone
      *
      * @return array   list of usernames with permission >0
      */
    function get_users () {
        
        $dbdo = DB_DataObject::Factory('zone_user');
        $dbdo->zone = $this->id;
        $dbdo->selectAdd();
        $dbdo->selectAdd("DISTINCT username as duser");
        $dbdo->find();

        //$sql = "SELECT DISTINCT username FROM zone_user WHERE zone='$this->id'";

        $users = array();
        while ($dbdo->fetch())
            $users[] = $dbdo->duser;

        return $users;
    } // get_users
    // }}}

// Static Methods
    // {{{ merge_zones()

    /**
      * Add one zone into another
      *
      * @param  int     ID of the first zone
      * @param  int     ID of the target zone (being added to by the first)
      * @return mixed   true on success, PEAR_Error on failure
      */
    function merge_zones($client_zone, $parent_zone) 
    {
        // Note:  If anyone ever needed transaction support, this would
        // be a PRIME example

        global $conf;

        // move the hosts and pointers over
        Zone::_update_map($client_zone, $parent_zone, 'host');
        Zone::_update_map($client_zone, $parent_zone, 'pointer');

        // remove all duplicate zone mappings
        Zone::_update_map($client_zone, $parent_zone, 'zone_domain', 'domain');
        Zone::_update_map($client_zone, $parent_zone, 'zone_range', 'range');
        Zone::_update_map($client_zone, $parent_zone, 'zone_user', 'username');
        Zone::_update_map($client_zone, $parent_zone, 'zone_workgroup', 'workgroup');
        Zone::_update_map($client_zone, $parent_zone, 'cname');
        Zone::_update_map($client_zone, $parent_zone, 'mx');

        // delete the old zone
        $zone = new Zone($client_zone);
        $result = $zone->delete();

        if (!$result) {
            return new PEAR_Error(
                    _("All references to the zone were deleted,").' '.
                    _("but the zone itself could not be deleted"));
        }

        return true;

    } //merge_zones
    // }}}

// Helper Functions
    // {{{ _delete_maps()

    /**
      * Deletes mappings to this zone, i.e. cname, mx, etc.
      *
      * Helps with purging
      *
      * @access private
      */
    function _delete_maps()
    {
        $maps = Zone::_maps();
        foreach ($maps as $map) {
            $dbdo = DB_DataObject::Factory("zone_$map");
            $dbdo->zone = $this->id;

            // handle each result individually for object_history's sake
            $dbdo->find();
            while ($dbdo->fetch()) {
                // not pretty, but the only generic way to store info
                $tmp = DB_DataObject::Factory("zone_$map");
                $tmp->get($dbdo->id);
                $tmp->delete();
            }
        }

        //manually deletes proper 'cname' table in the database
        $dbdo = DB_dataobject::factory('cname');
        $dbdo->zone = $this->id;
        $dbdo->find();
        while($dbdo->fetch())
            $dbdo->delete();

        //manually deletes proper 'mx' table in the database
        $dbdo = DB_dataobject::factory('mx');
        $dbdo->zone = $this->id;
        $dbdo->find();
        while($dbdo->fetch())
            $dbdo->delete();
        
    }

    // }}}
    // {{{ _update_map()

    /**
      * Moves all data one from zone_* mapping zone to another
      *
      * Helps ->merge_zones()
      *
      * @access private
      * @static
      * @param  int     ID of old zone
      * @param  int     ID of new, larger zone
      * @param  string  name of the map, i.e. for cname, this would be 'cname'
      * @param  string  the column to test for duplicate values in
      * @param  string  primary key of the mapping
      */
    function _update_map($old, $new, $map, $value='', $key='id')
    {
        /* Here's the process:
         *
         * If this maps to another table (i.e. $value is specified) {
             * 1. Compile a list of all zone_$map for both $old and $new zones
             * 2. Delete any duplicate entries for $value
         * }
         * 3. Perform an update so they all point to the new zone
         */

        if ($value) {

            // "UPDATE zone_$map SET zone='$new' WHERE zone='$old'

            /* 1. Compile a list of all zone_$map for both zones */

            // "SELECT $key,$value FROM zone_$map WHERE zone='$old' OR zone='$new'
            $dbdo = DB_DataObject::Factory($map);
            $dbdo->selectAdd();
            $dbdo->selectAdd($key);
            $dbdo->whereAdd("zone='$old'");
            $dbdo->whereAdd("zone='$new'");
            $dbdo->find();

            // array for storing all values found
            $results = array();

            // array for storing duplicates
            $duplicates = array();

            // find the duplicates
            while ($dbdo->fetch()) {
                if (!$results[$dbdo->$value]) {
                    $results[$dbdo->$value] = true;
                } else {
                    $duplicates[] = $dbdo->$key;
                }
            }

            /* 2. Remove duplicate entries */

            if ($duplicates) {

                foreach ($duplicates as $data) {

                    // $sql = "DELETE FROM zone_$map WHERE $key='$data'";
                    $dbdo = DB_DataObject::Factory($map);
                    $dbdo->$key = $data;

                    // handle one at a time for object_history's sake
                    $dbdo->find();
                    while ($dbdo->fetch()) {
                        $tmp = DB_DataObject::Factory($map);
                        $tmp->get($dbdo->$key, $data);
                        $tmp->delete();
                    }
                } //for each duplicate

            } //if duplicates

        }  // if $value

        /* 3. Perform an update so that all of these point to just the new zone */

        // $sql = "UPDATE $map SET zone='$new' WHERE zone='$old'";
        $dbdo = DB_DataObject::Factory($map);
        $dbdo->zone = $old;

        // one at a time for object_history's sake
        $dbdo->find();
        while($dbdo->fetch()) {
            $tmp = clone($dbdo);
            $tmp->zone = $new;
            $result = $tmp->update();

            // if we updated, add the object_history comment
            if ($result) {

                // add the update record
                $oldzone = new Zone($old);
                $newzone = new Zone($new);

                // set up the object history comment
                $msg = $tmp->_type() . ' '.  _("moved zones from");
                $msg .= " $oldzone->name " . _("to") . " $newzone->name";

                // write the history
                $tmp->write_history($msg, ACTION_MODIFY, '', $this->id);
            }
        }

    } // _update_map()

    // }}}
    // {{{ _maps()

    /**
      * Return a list of zone_* suffixes
      *
      * Helps with _update_maps() and _delete_maps()
      *
      * @access private
      * @static
      */
    function _maps()
    {
        return array('user', 'domain', 'range', 'workgroup');
    }

    // }}}

} // Zone

?>
