<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel  <bartel@osuosl.org>                            |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: DomainZoneForm.php 4084 2006-01-11 19:13:32Z bartel $

/**
 *  DomainZoneForm.php - contains the {@link DomainZoneForm} class
 *
 *  @author $Author:$
 *  @filesource
 *  @package    htdocs
 *  @subpackage forms
 */

require_once("MaintainForm.php");

/**
 *  This class is a form that is used to add or remove a domain to/from a zone.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class DomainZoneForm extends MaintainForm
{

//{{{   form_init_elements()

    /**
     *  This method is where we initialize all of the elements that are part of
     *  the form.
     */
    function form_init_elements()
    {
        $current = array();
        $potential = array();
        
        $add = new FEListBox(_("Add Domain to Zone"), false);
        $rem = new FEListBox(_("Remove Domain from Zone"), false);

        $add->set_list_data($this->_allZones());

        $rem->set_list_data($this->_DomainZones());
        
        $addbut = new FESubmitButton(_("Add"), _("Add"));
        $rembut = new FESubmitButton(_("Remove"), _("Remove"));

        $this->add_elements($add, $rem, $addbut, $rembut);
    }

//}}}
//{{{   form()

    /**
     *  This method is what generates the form for displaying to the screen.
     *
     *  @return DIVTag  $div    Div with the form contained inside of it.
     */
    function form()
    {
        $div = html_div("form");
        $table = html_table();

        $class = array(0=>"even",
                        1=>"odd");

        $temp = 1;

        //Add the form elements
        $tr = html_tr($class[++$temp%2]);
        $tr->add($this->element_label(_("Add Domain to Zone")), 
                $this->element_form(_("Add Domain to Zone")), 
                $this->add_action(_("Add")));
        $table->add($tr);


        $tr = html_tr($class[++$temp%2]);
        $tr->add($this->element_label(_("Remove Domain from Zone")), 
                $this->element_form(_("Remove Domain from Zone")), 
                $this->add_action(_("Remove")));
        $table->add($tr);

        $cont = container();
        $act = new FEHidden("action", "edit");
        $idnum = new FEHidden("id",$_REQUEST['id']);
        $this->add_elements($act, $idnum);
        
        $cont->add($this->element_form("action"),$this->element_form("id"));

        $div->add($table);
        $div->add($cont);

        return $div;
    }

//}}}
//{{{   form_action()

    /**
     *  This method is used to take the appropriate action given the form input.
     *
     *  @return Boolean True If successful
     */
    function form_action()
    {
        global $user;
        $res = true;

        switch(strtolower($this->get_action())) {

        case 'add':
            $zdobj = DB_DataObject::factory('zone_domain');
            $zdobj->domain = $_REQUEST['id'];
            $zdobj->query("BEGIN");
            $zdobj->zone = $this->get_element_value(_("Add Domain to Zone"));

            if(!$zdobj->insert()) {
                $zdobj->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"), _("Failed to add item to database"));
            } else {
                $zdobj->query("COMMIT");
                $zn = new Zone($_REQUEST['id']);
                $zn->write_history(_("Added Zone-Domain mapping"), ACTION_CREATE,
                                        $user->username, $_REQUEST['id']);
            }
            
            break;
            
        case 'remove':
            $zdobj = DB_DataObject::factory('zone_domain');
            $zdobj->id = $this->get_element_value(_("Remove Domain from Zone"));
            $zdobj->query("BEGIN");
            $zdobj->find();

            if(!$zdobj->delete()) {
                $zdobj->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"), _("Failed to remove item from database"));
            } else {
                $zdobj->query("COMMIT");
                $zn = new Zone($_REQUEST['id']);
                $zn->write_history(_("Remove Zone-Domain mapping"), ACTION_DELETE,
                                        $user->username, $zn->id);
            }

            break;
        }
        
        return $res;
    }

//}}}

//{{{ form_backend_validation()
    function form_backend_validation()
    {
        if(!strcasecmp(strtolower($this->get_action()),"add")) {
            $zone = $this->get_element_value(_("Add Domain to Zone"));
            $zd = DB_DataObject::Factory('zone_domain');
            $zd->zone = $zone;
            $zd->domain = $_REQUEST['id'];
            if ($zd->find())    {
                $this->add_error("Add Domain to Zone", "Domain already belongs to this zone");
                return false;
            }
        }
        return true;
    }
//}}}
//{{{ form_success()

    /**
     *  This method is executed upon a successful form action, in this case it
     *  takes them back to the edit page so they may continue.
     */
    function form_success()
    {
        header("Location: ".web_root()."domain.php?action=".$_REQUEST['action'].
                                                "&id=".$_REQUEST['id']);
    }

//}}}

//{{{ _allZones()
    /*
     *  This function returns a name=>id paired array of 
     *  all zones.
     *
     *
     */
    function _allZones()
    {
        $result = array();
        $obj = DB_DataObject::Factory('zone');
        $obj->query("BEGIN");
        $obj->orderBy('name ASC');
        $obj->find();
        
        while ($obj->fetch())
            $result[$obj->name] = $obj->id;
        $obj->query("COMMIT");
        return $result;        
    }
//}}}

//{{{ _DomainZones()
    /*
     *  This function returns a name=>id paired array of 
     *  all zones that are .
     *
     *
     */
    function _DomainZones()
    {
        $result = array();
        $zd = DB_DataObject::Factory('zone_domain');
        $zo = DB_DataObject::Factory('zone');
        $zd->domain = $_REQUEST['id'];
        $zo->joinAdd($zd);
        $zo->find();
        while ($zo->fetch())
            $result[$zo->name] = $zo->id;

        return $result;
    }
//}}}
}

?>
