<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: IPBlockEditForm.php 4655 2006-07-25 16:48:09Z schonstal $

/**
 *  IPBlockEditForm.php - this file contains the {@link IPBlockEditForm} class
 *
 *  @author $Author$
 *  @pacakge    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/IPBlockCreateForm.php");

/**
 *  This class is the form used to create an IPBlock
 *
 *  @package    htdocs 
 *  @subpackage forms
 */
 class IPBlockEditForm extends IPBlockCreateForm
 {

//{{{   form_init_data()

    /**
     *  This method sets the initiial form values to those of the block we're editing
     */
    function form_init_data()
    {
        $ipb = new IPBlock($_REQUEST['id']);
        $formary = $this->remap($ipb->toArray(),FORM);
       
        foreach($formary as $label=>$val)
            $this->set_element_value($label, $val);
    }

//}}}
//{{{   form_action()

    /**
     *  This method performs actions on the form values after validation has 
     *  successfully completed.
     *
     *  @return Boolean     True of action was successful
     */
    function form_action()
    {
        global $user;
        $res = true;
   
        switch(strtolower($this->get_action())) {

        case 'submit':
            $vals = $this->get_all_element_values();
            $dbdo = new IPBlock($_REQUEST['id']);
            $dbdo->query("BEGIN");
            
            $dbdo->setFrom($this->remap($vals,DBDO));
            $dbdo->enabled = ($vals[_('Enabled')] == 1 ? 1 : 0);

            if($dbdo->update() === false) {
                $dbdo->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"), _("Could not insert IP Block in the database"));
            } else {
                $dbdo->query("COMMIT");
                $dbdo->write_history(_("Updated IPBlock"), MODIFY, 
                                    $user->username, $_SESSION['zone']);
            }
            
            break;

        default:
            die_hard(_("Invalid form action"));

        }

        return $res;
    }

//}}}
//{{{   form_success()

    /** 
     *  This method displays a success message to the user if the action was
     *  successfully completed.
     *
     *  @return DIVTag  $div    Div with the message inside
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $div->add(_("IPBlock successfully updated!"));
        $indlink = html_a(web_root()."index.php", _("[Return to Home]"));
        $link = html_a(web_root()."ip.php", _("[Return to IP Block List]"));

        $div->add(html_br(), $indlink, $link);
        return $div;
    }

//}}}
//{{{   form_hidden()

    /**
     *  This is where the form hidden elements are created and then added
     *
     *  @return Container   A container of hidden elements
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));
    }

//}}}

}
 
?>
