<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain2/branches/2.6/class/htdocs/forms/NameserverCreateForm.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Zack Bartel <bartel@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// |            Josh Schonstal <schonstal@osuosl.org>                       |
// +------------------------------------------------------------------------+
// $Id: HostCreateForm.php 3966 2005-12-12 15:32:02Z bartel $

/**
 *  NameserverCreateForm.php - contains the {@link NameserverDisplayForm}
 *                   
 *
 *  @author $Author: bartel $
 *  @filesource
 *  @package    htdocs
 *  @subpackage forms
 */

define("DYNAMIC", 0);
define("STATICIP",1);

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This is the form for editing nss.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class NameserverCreateForm extends MaintainForm
{

//{{{   vars

    /**
     *  This var is an array that maps the form labels to the DBDO backend
     *  variables.  Relationship is form label => dbdo var.
     */
    var $map = array("Hostname"     =>"name",
                    "Domain"        =>"domain",
                    "Glue"          =>"glue",
                    );
//}}}

/*Form Content Method Overloading*/
//{{{   form_init_elements()

    /**
     *  This method makes the elements for the form.
     */
    function form_init_elements()
    {
        global $user, $id;
        
        $nsname = new FEText(_('Hostname'),true);
        $domain = new FEListBox(_("Domain"),true);
        $glue = new FEText(_('Glue'), false);
                 
        $domain->set_list_data($this->_domains());

        $this->add_elements($nsname, $domain, $glue);
        
        $this->set_action($_SERVER["SCRIPT_NAME"]);
    }

//}}}
//{{{   form_action()

    function form_action()
    {
        global $user;

        $res = false;
        switch($this->get_action())
        {
            case _("Submit"): //Create button got pressed
                $ns = new NS();
                $ns->query("BEGIN");  
                $vals = $this->remap($_POST, DBDO);
                //Lets set the nss values to the submitted ones.
                $ns->setFrom($vals);

                $valid = $ns->insert(); //attempt to insert


                if($valid === false) { //If validation failed due to error
                    $ns->query("ROLLBACK");
                    $this->add_error(_("ERROR"),_("Failed to create Nameserver!"));

                }
                else { //otherwise life is good
                    $ns->query("COMMIT");
                    //Make a record of the event
                    $ns->write_history(_('Nameserver Updated'), ACTION_CREATE, $user->username);
                    $res = true;
                }
             
                break;
            case _("Cancel"):
                $res = true;
                header("Location: ns.php?action=show");
                break;
            default:
                $this->add_error(_("ERROR"),_("Unrecognized action"));
                $res = false;
                break;
        }
        
        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method is executed when the form_action method returns true.
     *  In this case it redirects the user to the ns edit page with the new vals.
     */
    function form_success()
    {
        $div = html_div("success");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $nslink = html_a(web_root()."ns.php", _("[Return to Nameserver List]"));
        $div->add(_("Nameserver Successfully Updated"), html_br(), $indlink, $nslink);
        return $div;
    }


//}}}
//{{{   form_backend_validation()
    function form_backend_validation()
    {
        $ns = new NS($_REQUEST['id']); //make a ns
        $array = $this->get_all_element_values(); // get the form values
        $ns->setFrom($this->remap($array,DBDO)); //give them to the ns
        $valid = $ns->validate(); //validate the ns with new info
        
        $res = true; //assume true

        if(is_array($valid)) {
            $errors = $this->remap($valid, FORM); //Remap the errors to forms
            foreach($errors as $name=>$state)
                if($state !== true && $state != NULL)   {
                    //true or null (DB_DataObject) implies that part was valid
                    $res = false; //result is not valid form
                    $this->add_error($name,$state); //add an error
                }
       }
        return $res; //we're done
    }

//}}}


/*Helper Methods to generate arrays to populate the list boxes*/
//{{{   _domains()

    function _domains()
    {
        global $user;
        
        $domains = array();
       
        //The fun part about this is zone_domain doesn't know the name...
        //Thus we do a magical join operation because the domain table does.
        $zd = DB_DataObject::Factory('zone_domain');
        if(!$user->is_admin())//If they're a full admin we don't care about zones
            $zd->zone = $_SESSION['zone'];
        $dom = DB_DataObject::Factory('domain');
        $dom->joinAdd($zd);
        $dom->orderBy('name');

        $dom->find();

        while($dom->fetch()) {
            $domains[$dom->name] = $dom->domain;
        }

        return $domains;
    }

//}}}

}

?>
