<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/htdocs/forms/ObjectMoveZoneForm.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel  <bartel@osuosl.org>                            |
// +------------------------------------------------------------------------+
// $Id: ObjectMoveZoneForm.php 4503 2006-06-22 21:48:33Z bartel $

/**
 *  ObjectMoveZoneForm.php - contains the {@link HostDisplayForm}, {@link MoveHostForm}
 *                        and {@link ZoneChangeForm} and {@link DeleteConfirm} class
 *
 *  @author $Author: claym $
 *  @filesource
 *  @package    htdocs
 *  @subpackage forms
 */


/**
 *  This form allows a full admin to move an object across zones.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class ObjectMoveZoneForm extends MaintainForm
{

    var $object;
//{{{ constructor()
    /* Initialize the object
     *
     *
     *@param string $object     Object to be moved
     *
     */
    function ObjectMoveZoneForm($obj)   {
        $this->object = $obj;
    }   
//}}}
//{{{   form_init_elements()

    /**
     *  This method intializes the fields for the user
     */
    function form_init_elements()
    {
        global $user;
        $zones = $user->zones();        

        $zoneary = array();
        foreach ($zones as $var) {
            $dbdo = new Zone($var);
            $zoneary[$dbdo->getName()] = $dbdo->getId();
        }
        
        $zonelist = new FEListBox(_("Target Zone"));
        $zonelist->set_list_data($zoneary);
        
        $this->add_element($zonelist);
        $this->add_element(new FEHidden("action","edit"));
        $this->add_element(new FEHidden("id", $_REQUEST['id']));
        $this->set_required_marker(null);

        $this->set_action($SERVER["SCRIPT_NAME"]);
    }

//}}}
//{{{   form_init_data()

    /**
     *  This method sets the default value for the fields
     */
    function form_init_data()
    {
        $dbdo = new $this->object();
        $dbdo->id = $_REQUEST['id'];
        $dbdo->selectAdd();
        $dbdo->selectAdd('zone');
        $dbdo->find(1);

        $this->set_element_value(_('Target Zone'),$dbdo->zone);
    }

//}}}
//{{{   form()

    /**
     *  This method is what lays the form out on the screen.
     *
     *  @return DIVTag  $div    A div that contains the form object
     */
    function form()
    {

        $div = html_div("targetzone");
        $table = html_table();
        
        /*This is the title to the sub table*/
        $tr = html_tr("even");
        $span = html_span("subtitle");
        $span->add(_("Move {$this->object} to Another Zone"));
        $tr->add($span);
        $table->add($span);
        
        /*Here goes the the actual form element*/
        $tr = html_tr("odd");
        $tr->add($this->element_label(_('Target Zone')),$this->element_form(_('Target Zone')));
        $table->add($tr);

        /*This would be the local submit button for all your submission needs*/
        $tr = html_tr("even");
        $tr->add("&nbsp;",$this->add_action(_("Move {$this->object}")));
        $table->add($tr);
        $div->add($table);
        $div->add($this->element_form("action"));
        $div->add($this->element_form("id"));
        
        return $div;
    }

//}}}
//{{{   form_action()

    /**
     *  This method is what performs the action of performing the update
     *
     *  @return boolean     Update successful?
     */
    function form_action()
    {
        global $user;
        
        $obj = new $this->object($_REQUEST['id']);
        $obj->query("BEGIN");
        $oldzone = $obj->zone;
        $obj->setZone($this->get_element_value(_("Target Zone")));
        
        if($obj->update() === false) { //false means something bad happened
            $obj->query("ROLLBACK");
            $this->add_error("ERROR", "Update failed"); //That's not cool
            return false; //failure++
        } else {
            $obj->query("COMMIT");
            $obj->write_history(_("Moved {$this->object} from").' '.$oldzone.' '._("to").
                                ' '.$obj->zone, ACTION_MODIFY, $user->username,
                                $obj->zone);
        }
        return true; //we win!
    }

//}}}

//{{{   form_success()

    /**
     *  This method lets them know they were successful in creating the record
     *  and allows them to go back to the index or to the list.
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a("index.php", "[Return to Home]");
        $link = html_a("{$this->object}.php", "[Return to {$this->object} List]");
        $div->add(container("{$this->object} Record moved successfully!",html_br()));
        $div->add(container($indlink, $link));
        return $div; 
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This is the validation method.
     *
     *  @return boolean     Is the zone valid?
     */
    function form_backend_validation()
    {
        $obj = new $this->object($_REQUEST['id']);
        $obj->setZone($_REQUEST['Target_Zone']); //our new target zone

        $errors = $obj->validate(); //run validation to check for errors
        if($errors  !== true) { //if it wasn't true then it was the error array
            foreach($errors as $name=>$message) //iterate through
                if($message !== true)
                    $this->add_error($name, $message); //add the errors
            return false; //validation failed, see errors
        }
 
        return true; //validation successful
    }

//}}}

}

?>
