<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel   <bartel@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: ZoneMergeForm.php 3941 2006-01-22 23:20:27Z bartel $

/**
 *  ZoneMergeForm - This file contains the {@link ZoneMergeForm} class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class contains the form that is used by the users to merge zones
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class ZoneMergeForm extends MaintainForm
{

/*Methods*/
//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that are going to be used
     */
    function form_init_elements()
    {
        $data = $this->_zones();

        $source = new FEListBox(_("Source Zone"));
        $source->set_list_data($data);
        /* If we got here by selecting a zone in the zone list then we know the source */
        if (isset($_REQUEST['id'])) 
            $source->set_value($_REQUEST['id']);
        $dest = new FEListBox(_("Destination Zone"));
        $dest->set_list_data($data);
        
        $this->add_elements($source, $dest);
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method takes care of validation before the information is processed by
     *  the action method.
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        return true;
    }

//}}}
//{{{   form_action()

    /**
     *  This method processes the data after it passes validation
     *
     *  @return Boolean   True if processing was successful
     */
    function form_action()
    {

        global $user;
        $res = true;
    
        switch(strtolower($this->get_action())) {
        
        case 'submit':
            $vals = $this->get_all_element_values();
            $result = Zone::merge_zones($vals['Source_Zone'], $vals['Destination_Zone']);

            if (PEAR::isError($result)) {
                $this->add_error(_("ERROR"), $result->message);
                return false;
            }
            
            break;

        default:
            die_hard(_("Invalid form action"));
            break;
            
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a success page to the user
     * 
     *
     *  @return DIVTag  $div    A div that contains a success message
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $zonlink = html_a(web_root()."zone.php",_("[Return to Zone List]"));
        $div->add(_("Zone successfully merged"),html_br());
        $div->add(container($indlink,$zonlink));
        
        return $div;
    }

//}}}



    function _zones()
    {
        $ary = array();
        $zn = DB_DataObject::Factory('zone');
        //$zn->whereAdd('id != '.$_REQUEST['id']);
        $zn->orderBy('name'); 
        $zn->find();

        while ($zn->fetch())    {
            $ary[$zn->name] = $zn->id;
        }
        return $ary;
    }

}

?>
